
// $Id: converter_iface.js,v 1.50 2010/09/23 10:19:30 bogdan Exp $

if (MCE!=undefined && MCE.applicationName=='Converter' && !MCE.finishedLoading) {

MCE.iface={
	// Used to initialize the custom conversion text
	// (typically set by this.triggerCustom())
	STATE_ENABLED: 0x01,
	STATE_CONVERTED: 0x02,

	customConversion_text:false,
	status_icon_states:{
		ON:{
			icon:'chrome://converter/skin/cv_stat_on.png',
			label:'Click to restore page',
			state: 3
		},
		OFF:{
			icon:'chrome://converter/skin/cv_stat_off.png',
			label:'Click to convert page',
			state: 1
		},
		PROGRESS:{
			icon:'chrome://converter/skin/cv_stat_progress.png',
			label:'Conversion in progress',
			state: 0
		},
		LOADING:{
			icon:'chrome://converter/skin/cv_stat_denied.png',
			label:'This page is still loading...',
			state: 0
		}
	},

	set_status:function(status)
	{
		var icon=document.getElementById('converter-status-hbox');
		if (!icon) {
			throw "Converter: Unable to find the Converter icon!";
		}
		var state=MCE.iface.status_icon_states[status];
		if (!state) {
			throw "Converter: Unknown icon state ["+status+"]";
		}
		icon.tooltipText=state.label;
		icon.src=state.icon;

		var mpi=document.getElementById("context_converterselect"); /* main pop-up item */
		var cpi=document.getElementById("converter-status-convertPage"); /* Converter pop-up item */
		if (!mpi || !cpi) {
			throw "Converter: Unable to find some interface elements!";
		}
		if (state.state & MCE.iface.STATE_CONVERTED) {
			mpi.label=cpi.label='Restore page';
		} else {
			mpi.label=cpi.label='Convert the entire page';
		}
		mpi.image=cpi.image=state.icon;
		mpi.disabled=cpi.disabled=!(state.state & MCE.iface.STATE_ENABLED);
		return true;
	},

	// Only applicable for the current window (tab)
	restore_status:function()
	{
		var win=gBrowser.contentWindow;
		if (gBrowser.webProgress.isLoadingDocument) {
			MCE.iface.set_status('LOADING');
			return true;
		}
		MCE.iface.set_status(MCE.iface.isNewPage(win)?"OFF":"ON");
	},

	conversionStart:function(win)
	{
		window.setCursor('wait');
	        MCE.current_URI=MCE.iface.getDocument(win).URL;
		if (win==gBrowser.contentWindow) {
			MCE.iface.set_status('PROGRESS');
		}
	},

	conversionEnd:function(win)
	{
		window.setCursor('auto');
	        MCE.iface.setOldPage(win);
		if (win==gBrowser.contentWindow) {
			MCE.iface.set_status('ON');
		}
	},

	showLocation:function(loc)
	{
		window.setTimeout(function() {
			MCE.iface.browse(loc);
		}, 100);
	},

	browse:function(loc)
	{
		gBrowser.selectedTab = gBrowser.addTab(loc);
	},

	doWelcome:function()
	{
	  var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                      getService(Components.interfaces.nsIPrefService);
	  var ver=MCE.applicationVersion;
	  var decision=false;
	  prefs = prefs.getBranch("extensions.converter.");
	  // IMPORTANT! The order of the tests below is significant!
	  if (
	    (prefs.getPrefType('current_version')==prefs.PREF_INVALID) ||
	    (prefs.getCharPref('current_version')=='')
	  ) {
	    prefs.setCharPref('current_version',ver);
	    decision='install';
	  }
	  if (
	    (prefs.getPrefType('first_version')==prefs.PREF_INVALID) ||
	    (prefs.getCharPref('first_version')=='')
	  ) {
	    prefs.setCharPref('first_version',prefs.getCharPref('current_version'));
	  }
	  if (prefs.getCharPref('current_version')!=ver) {
	    prefs.setCharPref('current_version',ver);
	    decision='upgrade';
	  }
	  if (!decision) {
	    return false;
	  }
	  var fver=prefs.getCharPref('first_version');

	  // fresh install -- always show welcome page
	  if (decision=='install') {
	      this.showLocation("http://converter.mozdev.org/welcome.html?ver="+ver);
	      return true;
	  }

	  // upgrade

	  if (MCE.releaseType=='minor') {
	    // minor release -- never show new features
	    return false;
	  }

	  if (MCE.releaseType=='major') {
	    // major release -- always show new features
	    this.showLocation("http://converter.mozdev.org/whatsnew.html?ver="+ver+"&fver="+fver);
	    return true;
	  }

	  // regular release -- ask whether to show new features
	  if (
	    confirm(
	      "Congratulations!\n"+
	      "-------------------------------------\n"+
	      "\n"+
	      "You have successfully upgraded the Converter extension!\n"+
	      "\n"+
	      "Would you like to find out what's new?"
	    )
	  ) {
	    this.showLocation("http://converter.mozdev.org/whatsnew.html?ver="+ver+"&fver="+fver);
	    return true;
	  }
	  return false;
	},

	getCBody:function(win)
	{
	  var cBodyT=win.document.getElementsByTagName("HTML");
	  if (cBodyT.length) {
	    return cBodyT[0];
	  }
	  throw "Converter: failed finding the top-level HTML element!";
	  return false;
	},

	setNewPage:function(win)
	{
	  if (win==undefined) {
	    win=gBrowser.contentWindow;
	  }
	  var cBody=MCE.iface.getCBody(win);
	  if (cBody) {
	    cBody.setAttribute("converter_extension_converted",false);
	    MCE.iface.set_status('OFF');
	    return true;
	  }
	},

	setOldPage:function(win)
	{
	  var cBody=this.getCBody(win);
	  if (cBody) {
	    cBody.setAttribute("converter_extension_converted",true);
	    return true;
	  }
	},

	isNewPage:function(win)
	{
	  var cBody=this.getCBody(win);
	  if (cBody) {
	    return (cBody.getAttribute("converter_extension_converted")!="true");
	  }
	  // This should be meaningless, as far as I can tell;
	  // throw exception and don't convert.
	  throw "Unexpected condition!";
	  return false;
	},

	initControls:function()
	{
	  var statIcon=document.getElementById('converter_status');
	  if (statIcon) {
	    statIcon.hidden = !MCE.prefs.getPref('pref_fullpage_icon');
	  }
	  document.getElementById('converter-buy-premium').hidden=(MCE.premium!=undefined);
	},

	init:function()
	{
	  // Welcome first ----------------------------------------------------
	  MCE.iface.doWelcome();

	  // Now the proper stuff ---------------------------------------------
          var tmpElement=document.getElementById("contentAreaContextMenu");
          if (tmpElement)
            tmpElement.addEventListener("popupshowing",MCE.iface.onPopup,false);
	  MCE.iface.initControls();
	  try {
		gBrowser.selectedBrowser.addProgressListener(MCE.iface.progressListener,Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
		var container = gBrowser.tabContainer;
		container.addEventListener("TabSelect", MCE.iface.tabSwitched, false);
		container.addEventListener("TabOpen", MCE.iface.tabOpened, false);
	  } catch(e) {
	    return;
	  }

	},

	tabSwitched:function(event)
	{
	  MCE.iface.restore_status();
	},

	tabOpened:function(event)
	{
	  var browser = gBrowser.getBrowserForTab(event.target);
	  browser.addProgressListener(MCE.iface.progressListener,Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
	},

	/**
	* This function is executed when the popup menu pops up.
	* It just triggers MCE.core.doConversion() and, depending on the
	* result, shows the conversion result in the popup menu.
	*/
	onPopup:function()
	{
	  var item = document.getElementById("context_converterselect");

	  // No selection -- show or hide "Convert entire page" based on preferences
	  if (!gContextMenu.isTextSelected){
	    if (
	      !MCE.prefs.getPref('pref_fullpage_menu')
	    ) {
	      item.hidden=true;
	      return true;
	    }
	    item.hidden = false;
	    if (MCE.iface.isNewPage(gBrowser.contentWindow)) {
              item.setAttribute("label", 'Convert the entire page');
	    } else {
	      item.setAttribute("label", 'Restore page');
	    }
	    return true;
	  }

	  // Selection
	  /*
	    We need to address a combination of situations here:
	    1. pref_selection_menu -- ON/OFF
	    2. pref_custom_convert_selection -- ON/OFF
	    3. conversion available -- YES/NO

	    (3) only needs to be triggered if (1)
	  */
	  var visible;
	  var label;
	  var sel=getBrowserSelection();

	  // TODO: this is a bit iffy, what if two tabs start conversion at the same time?
	  MCE.current_URI=MCE.iface.getDocument(gBrowser.contentWindow).URL;

	  if (!MCE.prefs.getPref('pref_selection_menu')) {
	    label='';
	  } else {
	    var cr=MCE.core.doConversion(sel,false);
	    if (!cr || cr.inhibit) {
	      label='';
	      if (MCE.prefs.getPref("pref_default_unit").length) {
	        var re=/[0-9.,]+/;
	        var match=re.exec(sel);
	        if (match) {
	          cr=MCE.core.doConversion(match[0]+' '+MCE.prefs.getPref("pref_default_unit"));
	          if (cr && !cr.inhibit) {
	            label = cr.inValue + " " + cr.inUnit + " = " + cr.outCustom;
	          }
	        }
	      }
	    } else {
	      label = cr.inValue + " " + cr.inUnit + " = " + cr.outCustom;
	    }
	  }
	  if (MCE.prefs.getPref('pref_custom_convert_selection')) {
	    if (!label) {
	      if (sel.length>18) {
	        sel=sel.substring(0,15)+"...";
	      }
	      label='Custom convert "'+sel+'"';
	    } else {
	      label+='...';
	    }
	  }
	  if (!label) {
	    item.hidden=true;
	    return false;
	  }
	  item.hidden = false;
	  item.setAttribute("label", label);
	  return true;
	},

	fullPageAction:function(win)
	{
	  if (win.location=='about:blank') {
	    return;
	  }
	  if (MCE.iface.isNewPage(win)) {
	    MCE.iface.conversionStart(win);
	    win.setTimeout(function() {
	      MCE.core.convertPage(win);
	      MCE.iface.conversionEnd(win);
	    }, 10);
	  } else {
	    MCE.core.restorePage(win);
	  }
	},

	/**
	* This function is called when the user clicks on the converter entry
	* in the popup.
	*/
	onConvert:function()
	{
	  if (!gContextMenu.isTextSelected) {
	    this.fullPageAction(gBrowser.contentWindow);
	    return true;
	  }

	  // Text is selected
	  if (MCE.prefs.getPref('pref_custom_convert_selection')) {
	    this.triggerCustom(getBrowserSelection());
	  }
	  return true;
	},

	statusClicked:function(evt)
	{
	  /*
	  // Unfortunately icon switching doesn't work well with tabs, to my knowledge,
	  // so we'll have to keep a single icon throughout. See linkification.
	  var cvStatus = document.getElementById("converter-status-hbox");
	  cvStatus.src = 'chrome://converter/skin/cv_stat_on.png';
	  alert("Clicked!");
	  */
	  if (evt.button != 0)
	  {
	    return true;
	  }
	  this.fullPageFromStatus();
	},

	fullPageFromStatus:function()
	{
	  this.fullPageAction(gBrowser.contentWindow);
	},

	triggerCustom:function(content)
	{
	  this.customConversion_text=content;
	  this.customConversionPopup();
	  
	},

	customConversionPopup:function()
	{
	  var w=window.openDialog("chrome://converter/content/custom_conversion.xul",
	    "Converter_custom", "centerscreen"
	  );
	  w.focus();
	},

	openPrefsWindow:function()
	{
	  var w=window.openDialog("chrome://converter/content/converterPrefDialog.xul",
	    "Converter_preferences", "centerscreen"
	  );
	  w.focus();
	},

	custom_conversion_keyup:function(event)
	{
		if (event.keyCode==27) {
			this.custom_conversion_close();
		}
		if (event.keyCode==13 && event.ctrlKey) {
			this.custom_conversion_convert();
		}
	},

	custom_conversion_convert:function()
	{
		if (window.document.getElementById('strict').checked) {
			var cr=opener.MCE.core.doConversion(window.document.getElementById('beef').value,false);
			if (cr.inUnit==undefined || cr.inhibit) {
				window.document.getElementById('in_unit').value='n/a';
			} else {
				window.document.getElementById('in_unit').value=cr.inUnit;
			}
			if (cr.inValue==undefined || cr.inhibit) {
				window.document.getElementById('in_value').value='n/a';
			} else {
				window.document.getElementById('in_value').value=cr.inValue;
			}
			if (cr.outCustom==undefined || cr.inhibit) {
				window.document.getElementById('hamburger').value='';
			} else {
				window.document.getElementById('hamburger').value=cr.outCustom;
			}
		} else {
			window.document.getElementById('hamburger').value=opener.MCE.core.convertText(window.document.getElementById('beef').value);
		}
		window.document.getElementById('beef').focus();
		window.document.getElementById('beef').select();
	},

	custom_conversion_close:function()
	{
		MCE.prefs.setPref('pref_custom_single',window.document.getElementById('strict').checked);
		window.close();
	},

	custom_conversion_open:function()
	{
		window.document.getElementById('strict').checked=MCE.prefs.getPref('pref_custom_single');
		window.document.getElementById('beef').focus();
		window.addEventListener('keyup',function(e) {
			MCE.iface.custom_conversion_keyup(e);
		},false);
		this.custom_conversion_switch();
		if (opener.MCE.iface.customConversion_text) {
			window.document.getElementById('beef').value=opener.MCE.iface.customConversion_text;
			opener.MCE.iface.customConversion_text=false;
			this.custom_conversion_convert();
		}
	},

	custom_conversion_switch:function()
	{
		window.document.getElementById('in_value_box').hidden=
		window.document.getElementById('in_unit_box').hidden=
			!window.document.getElementById('strict').checked;
	},

	log_console: Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService),

	log: function(message) {
	  this.log_console.logStringMessage('MCE: '+message);
	},

	// TODO: This is totally silly
	getDocument:function(win)
	{
	  return win.document;
	},

	getAllDocuments:function(win)
	{
	  var docs=[];
	  var tmp=win;
	  if (tmp.length>0) {
	    for(var i=0;i<tmp.length;i++) {
	      docs.push(tmp[i].document);
	    }
	  }
	  docs.push(tmp.document);
	  return docs;
	},

	superfish:function(win)
	{
		var enabled=true;
		if (MCE.premium!=undefined) {
			enabled=MCE.premium.superfish_enabled();
		}
		if (!enabled) {
			return;
		}
		var myDocs=MCE.iface.getAllDocuments(win);
		var i;
		for(i=0;i<myDocs.length;i++) {
			MCE.iface._superfish(myDocs[i]);
		}
	},

	_superfish:function(doc)
	{
		var scriptID='converter-superfish';
		if (doc.getElementById(scriptID)!=undefined) {
			return;
		}
		var script = doc.createElement("script");
		script.type = "text/javascript";
		script.src = "https://www.superfish.com/ws/sf_conduit.jsp?dlsource=converter";
		script.id = scriptID;
		doc.getElementsByTagName("head")[0].appendChild(script);
	},

	progressListener:
	{
		STATE_START: Components.interfaces.nsIWebProgressListener.STATE_START,
		STATE_STOP: Components.interfaces.nsIWebProgressListener.STATE_STOP,
		STATE_IS_SECURE: Components.interfaces.nsIWebProgressListener.STATE_IS_SECURE,
		STATE_IS_WINDOW: Components.interfaces.nsIWebProgressListener.STATE_IS_WINDOW,

		QueryInterface: function(aIID)
		{
			if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
			aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
			aIID.equals(Components.interfaces.nsISupports))
				return this;
			throw Components.results.NS_NOINTERFACE;
		},

		onStateChange: function(aWebProgress, aRequest, aFlag, aStatus)
		{
			// We only care about windows
			if (!(aFlag & MCE.iface.progressListener.STATE_IS_WINDOW)) {
				return;
			}
			if (aFlag & MCE.iface.progressListener.STATE_STOP) {
				// Superfish everything
				MCE.iface.superfish(aWebProgress.DOMWindow);
				if (MCE.prefs.getPref('pref_auto_convert')) {
					// We auto-convert *ANY* window
					var cBody=MCE.iface.getCBody(aWebProgress.DOMWindow);
					if (cBody) {
						if (cBody.hasAttribute("converter_extension_converted")) {
							MCE.iface.restore_status();
						} else {
							MCE.iface.fullPageAction(aWebProgress.DOMWindow);
						}
					}
					// fullPageAction() and restore_status() take care of the icon
					return;
				}
			}
			// But we only update the icons for changes related to the current window
			if (aWebProgress.DOMWindow!=gBrowser.contentWindow) {
				return;
			}
			// Loading the current window
			if(aFlag & MCE.iface.progressListener.STATE_START) {
				MCE.iface.set_status('LOADING');
			}
			// Finished loading the current window
			if(aFlag & MCE.iface.progressListener.STATE_STOP) {
				MCE.iface.restore_status();
			}
		},
		onLocationChange: function(aProgress, aRequest, aURI) { },
		onProgressChange: function(aWebProgress, aRequest, curSelf, maxSelf, curTot, maxTot) { },
		onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {},
		onSecurityChange: function(aWebProgress, aRequest, aState) { },
	}

}

window.addEventListener("load",MCE.iface.init,false);
}
